<?php
class WPJAM_WEAPP_API{
	public static function get_wxacode($post_id, $format){
		$wxacode_url	= get_post_meta($post_id, 'weapp_wxacode', true);

		if(!$wxacode_url){
			$page_key	= $format == 'link' ? 'post_comments' : 'post';
			$path		= weapp_get_path($page_key, compact('post_id'));

			if($redirect_path = weapp_get_redirect_path($path)){
				$path	= $redirect_path;
			}

			$wxacode_url	= weapp_create_qrcode(['width'=>200,	'type'=>'wxacode',	'path'=>$path]);

			if(is_wp_error($wxacode_url)){
				return '';
			}

			update_post_meta($post_id, 'weapp_wxacode', $wxacode_url);
		}

		return wpjam_get_thumbnail($wxacode_url);
	}

	public static function get_config(){
		$config = [
			'statuses'		=> ['publish'=>'已发布', 'draft'=>'草稿', 'pending'=>'等待复审'],
			'categories'	=> wpjam_get_terms(['taxonomy'=>'category', 'output'=>'categories']),
			'category'		=> wpjam_get_terms(['taxonomy'=>'category', 'output'=>'categories']),
		];

		if(is_weapp()){
			$config['post_comments']	= post_type_supports('post', 'comments');
			$config['post_favs']		= post_type_supports('post', 'favs');

			$share_image	= weapp_get_setting('share_image');
			$bg_image		= weapp_get_setting('background_image');
			$default_bg		= plugins_url('static/bg.jpg', dirname(__FILE__));

			$config['share_image']		= $share_image ? wpjam_get_thumbnail($share_image, '500x400') : '';
			$config['background_image']	= $bg_image ? wpjam_get_thumbnail($bg_image, '800x0') : wpjam_get_thumbnail($default_bg);

			foreach(['post_list_ad', 'post_detail_ad'] as $ad_key){
				if(($ad = weapp_get_setting($ad_key)) && $ad['unit_id']){
					if(strpos($ad['unit_id'], 'adunit-') === false){
						$ad['unit_id']	= 'adunit-'.$ad['unit_id'];
					}

					if($ad_key == 'post_list_ad'){
						$position	= ((int)$ad['position']) ?: 1;
					}

					if($ad['type'] == 'banner'){
						$ad	= wp_array_slice_assoc($ad, ['type', 'unit_id']);
					}elseif($ad['type'] == 'video'){
						$ad	= wp_array_slice_assoc($ad, ['type', 'unit_id', 'theme']);
					}else{
						$ad['grid_count']	= (int)$ad['grid_count'];
					}

					if($ad_key == 'post_list_ad'){
						$ad['position']	= $position;
					}

					$config[$ad_key]	= $ad;
				}else{
					$config[$ad_key]	= new stdClass;
				}
			}
		}

		return apply_filters('wpjam_blog_config', $config);
	}

	public static function register_api($json){
		if($json == 'blog.config'){
			wpjam_register_api('blog.config', [
				'modules'	=> ['type'=>'config']
			]);
		}elseif($json == 'blog.me'){
			wpjam_register_api('blog.me', [
				'title'			=> '个人中心',
				'page_title'	=> '个人中心',
				'share_title'	=> '个人中心',
			]);
		}elseif($json == 'category.list'){
			wpjam_register_api('category.list', [
				'title'			=> '专题',
				'page_title'	=> '专题',
				'share_title'	=> '专题',
				'modules'	=> [
					[
						'type'	=> 'taxonomy',
						'args'	=> ['taxonomy'=>'category',	'output'=>'categories']
					],
					[
						'type'	=> 'taxonomy',
						'args'	=> ['taxonomy'=>'category']
					]
				]
			]);
		}elseif($json == 'upload.media'){
			if(did_action('weapp_loaded' ) && weapp_get_setting('upload_attachment')){
				$upload_module	= [
					'type'	=> 'post_type',
					'args'	=> ['media'=>'media', 'action'=> 'upload']
				];
			}else{
				$upload_module	= [
					'type'	=> 'media',
					'args'	=> ['media'=>'media']
				];
			}

			wpjam_register_api('upload.media', [
				'title'		=> '上传图片',
				'auth'		=> true,
				'modules'	=> [$upload_module]
			]);
		}elseif($json == 'module.page'){	// 免费小程序兼容接口
			if(did_action('wpjam_module_loaded')){
				return;
			}

			$counts	= wp_count_posts('post');

			$module_page	= [
				'name'				=> get_option('blogname'),
				'add_to_favorites'	=> true,
				'infinite_loading'	=> false,
				'padding'			=> 0,
				'border_radius'		=> 0,
				'bg_color'			=> '',
				'bg_img'			=> '',
				'icons'				=> [],
				'search_box'		=> ['total'=>(int)$counts->publish],
			];

			$modules		= [];

			if($sticky_posts = get_option('sticky_posts')){
				$sticky	= wpjam_get_post($sticky_posts[0],['thumbnail_size'=>'900x600']);

				$modules[]	= [
					'type'		=> 'sticky',
					'status'	=> 'publish',
					'title'		=> '置顶文章',
					'setting'	=> [
						'bg_img'			=> $sticky['thumbnail'],
						'title_display'		=> false,
					],
					'content'	=> [$sticky]
				];
			}

			$_query	= wpjam_query(['posts_per_page'=>10, 'post_type'=>'post', 'post_status'=>'publish']);

			if($_query->have_posts()){
				$content	= array_map(function($post) use($setting){ return wpjam_get_post($post->ID); }, $_query->posts);
			}else{
				$content	= [];
			}

			$modules[] = [
				'type'		=> 'post',
				'status'	=> 'publish',
				'title'		=> '最新文章',
				'setting'	=> [
					'orderby'			=> 'post_date',
					'number' 			=> 10,
					'total_pages'		=> $_query->total_pages,
					'next_cursor'		=> ($_query->max_num_pages > 1 ? end($content)['timestamp'] : 0),
					'more'				=> true,
					'title_display'		=> false,
				],
				'content'	=> $content
			];

			wpjam_register_api('module.page', [
				'page_title'	=> get_option('blogname'),
				'share_title'	=> get_option('blogname'),
				'share_image'	=> weapp_get_setting('share_image') ? wpjam_get_thumbnail(weapp_get_setting('share_image'), '500x400') : '',
				'modules'		=> [[
					'type'	=> 'other',
					'args'	=> ['module_page'=>$module_page, 'modules'=>$modules]
				]]
			]);
		}
	}

	public static function filter_post_json($post_json, $post_id, $args){
		if($post_json['post_type'] == 'post'){
			$post_list_mode	= $post_detail_style = 0;

			if(weapp_get_appid()){
				$post_list_mode		= (int)weapp_get_setting('post_list_mode');
				$post_detail_style	= (int)weapp_get_setting('post_detail_style');
			}

			$post_json['mode']	= (int)wpjam_get_metadata('post', $post_id, 'mode', $post_list_mode);
			$post_json['style']	= (int)wpjam_get_metadata('post', $post_id, 'style', $post_detail_style);

			if(is_singular('post')){
				// $post_json['thumbnail']		= wpjam_get_post_thumbnail_url($post_id, [1000,680]);
				$post_json['poster']		= wpjam_get_post_thumbnail_url($post_id, [600,400]);
				$post_json['share_image']	= wpjam_get_post_thumbnail_url($post_id, [500,400]);

				if(is_weapp()){
					$post_json['wxacode']	= self::get_wxacode($post_id, $post_json['format']);
				}

				$related_query	= wpjam_get_related_posts_query(3);

				$post_json['related']	= wpjam_parse_query($related_query, ['number'=>3,	'size'=>[900, 360]]);
			}elseif(is_search() || (isset($args['action']) && $args['action'] == 'fav.list')){
				$post_json['thumbnail']	= wpjam_get_post_thumbnail_url($post_id, [900, 360]);
			}elseif($post_json['mode'] == 1 && empty($args['thumbnail_size'])){
				$post_json['thumbnail']	= wpjam_get_post_thumbnail_url($post_id, [300, 300]);
			}
		}

		return $post_json;
	}

	public static function filter_term_json($term_json, $term_id){
		if($term_json['taxonomy'] == 'category'){
			$term_json['thumbnail']	= wpjam_get_term_thumbnail_url($term_id, '480x640') ?: '';
		}elseif($term_json['taxonomy'] == 'module_page'){
			$term_json['module_excerpt']	= (int)get_term_meta($term_id, 'module_excerpt', true);
			$term_json['add_to_favorites']	= (bool)get_term_meta($term_id, 'add_to_favorites', true);
			$term_json['infinite_loading']	= (bool)get_term_meta($term_id, 'infinite_loading', true);

			$search_box	= get_term_meta($term_id, 'search_box', true);

			if(is_admin()){
				$term_json['search_box']	= $search_box;
			}elseif($search_box){
				$term_json['search_box']	= ['total'=>(int)(wpjam_query([
					'post_type'				=> 'post',
					'no_found_rows'			=> false,
					'posts_per_page'		=> 1,
					'module_query'			=> true,
					'update_post_meta_cache'=> false,
					'update_post_term_cache'=> false,
				])->found_posts)];
			}
		}

		return $term_json;
	}

	public static function filter_wpjam_json($response, $api_setting, $wpjam_json){
		if($wpjam_json == 'post.list'){
			if(did_action('wpjam_module_loaded') && !empty($_GET['module_id'])){
				$category_id	= $response['module_setting']['cat'] ?? 0;
			}elseif(is_home()){
				$response['share_image']	= weapp_get_setting('share_image') ? wpjam_get_thumbnail(weapp_get_setting('share_image'), '500x400') : ''; 

				// if(empty($_GET['cursor'])){
				// 	$sticky_posts	= get_option('sticky_posts');
				// 	if($sticky_posts){
				// 		$response['sticky']	= wpjam_get_post($sticky_posts[0],['thumbnail_size'=>'1000x680']);
				// 		$response['bg_img']	= $response['sticky']['thumbnail'];
				// 	}else{
				// 		$response['sticky']	= null;
				// 		$response['bg_img']	= '';
				// 	}

				// 	$response['page_title']	= $response['share_title']	= get_option('blogname');
				// }
			}elseif(is_category()){
				$response['share_image']	= wpjam_get_term_thumbnail_url(null,'500x400') ?: '';

				if(!empty($_GET['category_id'])){
					$category_id	= (int)$_GET['category_id'];
				}elseif(!empty($_GET['cat']) || !empty($_GET['category_name'])){
					$category_id	= (int)$_GET['cat'];
				}elseif(!empty($_GET['category_name'])){
					if($category = get_category_by_slug($_GET['category_name'])){
						$category_id	= $category->term_id;
					}
				}
			}

			$category_args = [
				'taxonomy'		=> 'category',
				'output'		=> 'categories',
				'hide_empty'	=> 0
			];

			if(!empty($category_id)){
				$category = get_term($category_id, 'category');

				if($category && !is_wp_error($category)){
					// $category_args['parent']	= $category->parent ?: $category_id;
				}
			}

			$response['categories']	= wpjam_get_terms($category_args, get_taxonomy('category')->levels) ?: [];
		}elseif($wpjam_json == 'post.get'){
			$response['bg_img']	= $response['post']['thumbnail'];
		}elseif($wpjam_json == 'category.list'){
			$response['share_image']	= weapp_get_setting('share_image') ? wpjam_get_thumbnail(weapp_get_setting('share_image'), '500x400') : ''; 
		}elseif($wpjam_json == 'blog.me'){
			$response['config']				= self::get_config();
			$response['background_image']	= $response['config']['background_image'];
		}elseif($wpjam_json == 'module.page'){
			$response['config']	= self::get_config();
		}

		return $response;
	}
}

wpjam_register_config(['WPJAM_WEAPP_API', 'get_config']);

add_action('wpjam_api',	['WPJAM_WEAPP_API', 'register_api']);

add_filter('wpjam_post_json',	['WPJAM_WEAPP_API', 'filter_post_json'], 99, 3);
add_filter('wpjam_term_json',	['WPJAM_WEAPP_API', 'filter_term_json'], 10, 2);
add_filter('wpjam_json',		['WPJAM_WEAPP_API', 'filter_wpjam_json'], 10, 3);